import urllib3
import numpy as np

from backend.landing import Landing


class LandingClient:
    def __init__(self, address: str):
        self.address = address.strip('/')
        self.http_pm = urllib3.PoolManager()

    def parse_status_response(self, data: bytes):
        fields = data.decode("ASCII").split('\r\n')
        landing_id = int(fields[0])
        pos = np.array(list(map(float, fields[1].split(' '))))
        vel = np.array(list(map(float, fields[2].split(' '))))
        fuel = float(fields[3])
        lspos = np.array(list(map(float, fields[4].split(' '))))
        return landing_id, pos, vel, fuel, lspos, np.array([0, 0, 0])

    def start(self):
        start_req = self.http_pm.request('GET', "%s/%s" % (self.address, "start"))
        if start_req.status != 200:
            return False
        return Landing(*self.parse_status_response(start_req.data))

    def set_engine_status(self, landing: Landing, right: bool, left: bool, up: bool):
        status_req = self.http_pm.request('GET', "%s/%s?id=%d&doprava=%d&doleva=%d&nahoru=%d" %
                                          (self.address, "stav", landing.id, right, left, up))
        if status_req.status != 200:
            return False
        if status_req.data == b'Krach!':
            landing.status = Landing.Status.LAND_FAILURE
            return landing
        elif status_req.data == b'Zdar!':
            landing.status = Landing.Status.LAND_SUCCESS
            return landing
        landing = Landing(*self.parse_status_response(status_req.data))
        landing.engines = np.array(list(map(int, [right, left, up])))
        return landing