import numpy as np
from enum import Enum


class Landing:
    class Status(Enum):
        AIRBONE = 0
        LAND_SUCCESS = 1
        LAND_FAILURE = 2

    def __init__(self, id: int, pos: np.ndarray, velocity: np.ndarray, fuel: float, lspos: np.ndarray,
                 engines: np.ndarray):
        self.status = self.Status.AIRBONE
        self.id = id
        self.pos = pos
        self.v = velocity
        self.fuel = fuel
        self.lspos = lspos
        self.engines = engines  # R, L,  U

    def update_pos(self, t):
        pos_eng_vect = np.array([
            6 * (self.engines[0] - self.engines[1]),
            -7 + 12 * self.engines[2]
        ])
        self.pos += (self.v * t) + pos_eng_vect * 0.5 * t ** 2

    def update_v(self, t):
        vel_eng_vect = np.array([
            6 * self.engines[0] - 6 * self.engines[1],
            -7 + 12 * self.engines[2]
        ])
        self.v = self.v + vel_eng_vect * t

    def update_fuel(self, t):
        if self.fuel <= 0:
            self.fuel = 0
        else:
            self.fuel = self.fuel - 0.5 * (self.engines[1] + self.engines[0] + 2 * self.engines[2]) * t

    def tick(self, t):
        if self.status == self.Status.AIRBONE:
            self.update_pos(t)
            self.update_v(t)
            self.update_fuel(t)
            if (self.pos[1] - 20) < 0:
                if ((self.pos[0] - 20) >= self.lspos[0]) and ((self.pos[0] + 20) <= self.lspos[1]):  # LS Confirmation
                    if (abs(self.v[0]) < 5) and (abs(self.v[1]) < 5):  # vL confirmation
                        self.status = self.Status.LAND_SUCCESS
                        return
                self.status = self.Status.LAND_FAILURE
