#!/usr/bin/env python3.5
import datetime

from tkinter import *
from backend.client import LandingClient
from backend.landing import Landing


class LandingFrontend(Tk):
    def __init__(self, landing_client: LandingClient, *args, **kwargs):
        Tk.__init__(self, *args, **kwargs)
        self.init_widgets()
        self.init_input()
        self.landing_client = landing_client
        self.landing = landing_client.start()
        self.title("Přistání na měsíci")

    def handle_up(self, _):
        self.landing = \
            self.landing_client.set_engine_status(self.landing,
                                                  self.landing.engines[0],
                                                  self.landing.engines[1],
                                                  False if self.landing.engines[2] else True)

    def handle_right(self, _):
        self.landing = \
            self.landing_client.set_engine_status(self.landing,
                                                  False if self.landing.engines[0] else True,
                                                  False,
                                                  self.landing.engines[2])

    def handle_left(self, _):
        self.landing = \
            self.landing_client.set_engine_status(self.landing,
                                                  False,
                                                  False if self.landing.engines[1] else True,
                                                  self.landing.engines[2])

    def init_input(self):
        self.bind("<Up>", self.handle_up)
        self.bind("<Right>", self.handle_right)
        self.bind("<Left>", self.handle_left)

    def update_labels(self):
        if self.landing.status == Landing.Status.LAND_FAILURE:
            self.title('Neuspech')
        elif self.landing.status == Landing.Status.LAND_SUCCESS:
            self.title('Uspech')
        self.postext.set('Pozice: [%.2f, %.2f]' % (self.landing.pos[0], self.landing.pos[1]))
        if (abs(self.landing.v[0]) > 5) or (abs(self.landing.v[1]) > 5):
            self.veltextw.config(fg='red')
        else:
            self.veltextw.config(fg='green')
        self.veltext.set('Rychlost: [%.2f, %.2f]' % (self.landing.v[0], self.landing.v[1]))
        if self.landing.fuel <= 0:
            self.fueltextw.config(fg='red')
        self.fueltext.set('Palivo: %.2f' % self.landing.fuel)
        self.rengtextw.config(fg='green' if self.landing.engines[0] else 'black')
        self.lengtextw.config(fg='green' if self.landing.engines[1] else 'black')
        self.uengtextw.config(fg='green' if self.landing.engines[2] else 'black')

    def redraw_landing(self):
        self.canvas.delete(ALL)
        self.canvas.create_image((self.landing.pos[0] * 2, (self.canvas.winfo_height() - self.landing.pos[1])),
                                 image=self.lander_img)
        self.canvas.create_line(self.landing.lspos[0] * 0.5, self.canvas.winfo_height(), self.landing.lspos[1] * 1.5,
                                self.canvas.winfo_height(), width=10)

    def init_widgets(self):
        self.canvas = Canvas(self, height=400, width=400)
        self.canvas.pack(fill=BOTH, expand=True, side='bottom')

        self.postext = StringVar()
        self.postextw = Label(self, textvariable=self.postext, )
        self.postextw.pack(side='top')

        self.veltext = StringVar()
        self.veltextw = Label(self, textvariable=self.veltext)
        self.veltextw.pack(side='top')

        self.fueltext = StringVar()
        self.fueltextw = Label(self, textvariable=self.fueltext)
        self.fueltextw.pack(side='top')

        self.rengtext = StringVar()
        self.rengtext.set('Doprava')
        self.rengtextw = Label(self, textvariable=self.rengtext)
        self.rengtextw.pack(side='top')

        self.lengtext = StringVar()
        self.lengtext.set('Doleva')
        self.lengtextw = Label(self, textvariable=self.lengtext)
        self.lengtextw.pack(side='top')

        self.uengtext = StringVar()
        self.uengtext.set('Nahoru')
        self.uengtextw = Label(self, textvariable=self.uengtext)
        self.uengtextw.pack(side='top')

        self.lander_img = PhotoImage(file='lunar_lander.png')
        self.lander_img = self.lander_img.subsample(5, 5)

    def mainloop(self, n=0):
        time_last = datetime.datetime.now()
        while True:
            self.update_idletasks()
            self.update()

            time_now = datetime.datetime.now()
            time_delta = time_now - time_last
            s_delta = time_delta.total_seconds()
            time_last = time_now

            self.landing.tick(s_delta)
            self.update_labels()
            self.redraw_landing()

try:
    landing_client = LandingClient('http://127.0.0.1:8000')
    fend = LandingFrontend(landing_client)
    fend.mainloop()
except TclError:
    pass  # Prevents error message on exit
